﻿#ifndef CEDITOR_H
#define CEDITOR_H

#include "../common/sources/Map.h"
#include "bridge.h"
#include "clipboard.h"
#include <QUndoStack>
#include <QQuickItem>

#define CURSOR_COLOR    QColor(0, 255, 0)
#define SELECTION_COLOR QColor(0, 255, 255)

struct CTileData
{
    QString name;
    QString image;
};

struct CWallData
{
    QString name;
    QString image;
};

class CEditor
{
public:
    CEditor();

    QImage      displayMap(CMap& map, CVec2 mousePos, ETabIndex tabIndex);
    EWallSide   getWallSide(CVec2 localPos);
    EWallSide   getWallSideAbs(CVec2 pos);
    bool        isSelectSingleWall();
    bool        isSelectSingleObject();
    CRect       getSelectionRect();
    QString     getPositionString(CVec2 mousePos, ETabIndex tabIndex);
    void        useTool();
    void        toolCopy();
    void        toolCut();
    void        unselect();
    void        hideSelectionInfos();
    void        initLists();
    void        deleteParamItems(std::vector<QQuickItem*>* list);
    void        addTileParamsItems(uint8_t type);
    void        addWallParamsItems(uint8_t type);
    void        addObjParamsItems();

    QUndoStack* mUndoStack;
    CClipboard  mClipboard;
    CVec2       mSelectStart;
    CVec2       mSelectEnd;

    std::vector<CTileData>  tilesDatas;
    std::vector<CWallData>  wallsDatas;
    std::vector<QQuickItem*>    mTileParamItems;
    std::vector<QQuickItem*>    mWallParamItems;
    std::vector<QQuickItem*>    mObjParamItems;

    void        readTilesDB(QString fileName);
    void        readWallsDB(QString fileName);
    void        readOnlyNamesDB(QString fileName, QString tagName, QStringList* list);

private:
    CRect       getSelectionRectPixel();
    void        toolDraw();
    void        toolSelect();
    void        toolPaste();
    void        copyData(CRect rect);
    void        displayWall(QImage* image, CVec2 pos, EWallSide side, int angle);
    QQuickItem* loadQml(QString file, QQuickItem* parent);
    void        addLabel(QQuickItem* parent, QString text, std::vector<QQuickItem*>* list);
    void        addComboBox(QQuickItem* parent, QStringList& values, std::vector<QQuickItem*>* list, int id, int index);
    void        addTextField(QQuickItem* parent, QString text, std::vector<QQuickItem*>* list, int id);
    void        addScriptField(QQuickItem* parent, QString text, std::vector<QQuickItem*>* list, int id);
    void        addCheckBox(QQuickItem* parent, bool isChecked, std::vector<QQuickItem*>* list, int id);
    void        addButton(QQuickItem* parent, QString text, std::vector<QQuickItem *> *list, int id);
};

extern CEditor  editor;

#endif // CEDITOR_H
