﻿#include "objects.h"
#include <QtXml/qdom.h>
#include <QFile>
#include <QDebug>
#include "../common/sources/system/filecache.h"
#include "../common/sources/system/cgraphics2d.h"
#include "interface.h"
#include "player.h"
#include "walls.h"
#include "game.h"

CObjects objects;

CObjects::CObjects()
{

}

//---------------------------------------------------------------------------------------------
void    CObjects::readObjectsDB()
{
    QDomDocument doc;

    QFile f("databases/items.xml");
    f.open(QIODevice::ReadOnly);
    doc.setContent(&f);
    f.close();

    QDomElement root = doc.documentElement();
    QDomNode    itemNode = root.firstChild();

    while(!itemNode.isNull())
    {
        if (itemNode.isElement())
        {
            QDomElement item = itemNode.toElement();

            if (item.tagName() == "item")
            {
                CObjectInfo object;
                object.name = interface.getText(item.attribute("name").toLocal8Bit().constData());

                QDomElement itemInfo = item.firstChild().toElement();

                while(!itemInfo.isNull())
                {
                    if (itemInfo.tagName() == "category")
                    {
                        object.category = itemInfo.text();
                    }
                    else if (itemInfo.tagName() == "floor_image")
                    {
                        object.floorImage = QString("gfx/3DView/items_floor/") + itemInfo.text();
                    }

                    itemInfo = itemInfo.nextSibling().toElement();
                }

                objects.mObjectInfos.push_back(object);
            }
        }
        itemNode = itemNode.nextSibling();
    }
}

//---------------------------------------------------------------------------------------------
CVec2 CObjects::getObjectPos(CVec2 tablePos)
{
    CVec2   pos;
    pos.x = 250 * (tablePos.x - 4.5) / (8.5 - tablePos.y) + 112;
    pos.y = 310 / (8.5 - 0.9 * tablePos.y) + 66;
    return pos;
}

//---------------------------------------------------------------------------------------------
void CObjects::drawObjectsStack(QImage* image, CVec2 mapPos, CVec2 tablePos)
{
    if (tablePos.y < 1 ||
        tablePos.y > (WALL_TABLE_HEIGHT - 1) * 2)
        return;

    CObjectStack*   stack = map.findObjectsStack(mapPos);

    if (stack != NULL)
    {
        for (size_t i = 0; i < stack->getSize(); ++i)
        {
            int type = stack->getObject(i).getType();

            if (type != 0)
            {
                // récupère l'image de l'objet
                CObjectInfo object = mObjectInfos[type - 1];
                QImage  objectImage = fileCache.getImage(object.floorImage.toLocal8Bit().constData());

                // récupère la position de l'objet et ajoute une valeur pseudo-aléatoire
                CVec2   pos = getObjectPos(tablePos);
                pos.x += ((mapPos.x - mapPos.y + i) * 53) % 7 - 3;
                pos.y += (i < 4 ? i : 3);

                // calcule la taille par rapport à la position de référence (la plus proche)
                CVec2   pos0 = getObjectPos(CVec2(WALL_TABLE_WIDTH, (WALL_TABLE_HEIGHT - 1) * 2));
                CVec2   pos1 = getObjectPos(CVec2(WALL_TABLE_WIDTH, tablePos.y));
                float   scale = (float)(pos1.x - 112) / (float)(pos0.x - 112);

                // redimensionne l'objet dans une image temporaire
                QImage  scaledObject(objectImage.size() * scale, QImage::Format_ARGB32);
                scaledObject.fill(QColor(0, 0, 0, 0));
                graph2D.drawImageScaled(&scaledObject, CVec2(), objectImage, scale);

                // assombrit l'objet en fonction de sa distance
                float shadow = ((WALL_TABLE_HEIGHT - 1) * 2 - tablePos.y) * 0.13f;
                graph2D.darken(&scaledObject, shadow);

                // dessine l'objet à l'écran
                pos -= CVec2(scaledObject.size().width() / 2, scaledObject.size().height() - 1);
                graph2D.drawImage(image, pos, scaledObject, 0, false, QRect(0, 33, MAINVIEW_WIDTH, MAINVIEW_HEIGHT));
            }
        }
    }
}

//---------------------------------------------------------------------------------------------
void CObjects::drawBackRow(QImage* image, CVec2 mapPos, CVec2 tablePos)
{
    CVec2   pos1, pos2;

    switch (player.dir)
    {
        case 0: // haut
            pos1 = CVec2(0, 0);
            pos2 = CVec2(1, 0);
            break;

        case 1: // gauche
            pos1 = CVec2(0, 1);
            pos2 = CVec2(0, 0);
            break;

        case 2: // bas
            pos1 = CVec2(1, 1);
            pos2 = CVec2(0, 1);
            break;

        case 3: // droite
            pos1 = CVec2(1, 0);
            pos2 = CVec2(1, 1);
            break;
    }

    drawObjectsStack(image, mapPos * 2 + pos1, tablePos * 2 + CVec2(0, 0));
    drawObjectsStack(image, mapPos * 2 + pos2, tablePos * 2 + CVec2(1, 0));
}

//---------------------------------------------------------------------------------------------
void CObjects::drawFrontRow(QImage* image, CVec2 mapPos, CVec2 tablePos)
{
    CVec2   pos1, pos2;

    switch (player.dir)
    {
        case 0: // haut
            pos1 = CVec2(0, 1);
            pos2 = CVec2(1, 1);
            break;

        case 1: // gauche
            pos1 = CVec2(1, 1);
            pos2 = CVec2(1, 0);
            break;

        case 2: // bas
            pos1 = CVec2(1, 0);
            pos2 = CVec2(0, 0);
            break;

        case 3: // droite
            pos1 = CVec2(0, 0);
            pos2 = CVec2(0, 1);
            break;
    }

    drawObjectsStack(image, mapPos * 2 + pos1, tablePos * 2 + CVec2(0, 1));
    drawObjectsStack(image, mapPos * 2 + pos2, tablePos * 2 + CVec2(1, 1));
}
